"""The Python Session Control Library is used to control iTest sessions to enable quick and easy interaction with
devices from a Python script.

Target devices/APIs may be defined either on the fly, or via session profiles defined in iTest projects.

The library may operate in either one of these modes:

    * Connected to an iTest GUI or Velocity agent instance.
    * Standalone on a workstation. In this mode, the library runs its own instance of the agent, which should be
    installed locally.
"""
